/* meko
 * Copyright (C) 2001 Scott James Remnant <scott@netsplit.com>.
 * All Rights Reserved.
 *
 * chat_interface.h
 * -- 
 * $Id: chat_interface.h,v 1.7 2001/06/17 16:22:40 scott Exp $
 * 
 * This file is distributed according to the GNU General Public
 * License.  For full details, read the top of 'main.c' or the
 * file called COPYING that was distributed with this code.
 */

#ifndef _MEKO_CHAT_INTERFACE_H
#define _MEKO_CHAT_INTERFACE_H

/* Required includes */
#include <stdarg.h>
#include <time.h>
#include <glib.h>
#include <gtk/gtk.h>
#include "say2_types.h"

/* Types of update we can perform */
typedef enum {
	CHAT_UPDATE_CLOSE,
	CHAT_UPDATE_SIGNON,
	CHAT_UPDATE_USER,
	CHAT_UPDATE_SIGNOFF
} ChatUpdateType;

/* Recipient types accepted by chat interfaces */
typedef enum {
	CHAT_RECIPTYPE_NONE,
	CHAT_RECIPTYPE_ANY,
	CHAT_RECIPTYPE_USER,
	CHAT_RECIPTYPE_CHAN
} ChatRecipType;

/* Defined in a sec... */
typedef struct _chat_interface ChatInterface;

/* Title change callback */
typedef void (*ChatTitleFn)(ChatInterface *, gpointer);

/* Structure to hold pointers to the various widgets in a chat interface
 * that we care about
 */
struct _chat_interface {
	/* What?  No ref_count?
	 *
	 * ChatInterface's are handled much more like GtkWidget's and, in fact,
	 * if we destroy top_vbox the ChatInterface will go with it.  When I
	 * get to that Chapter in the book, I may re-implement ChatInterface
	 * as an actual GtkWidget (although maybe they're too specialised to
	 * really be a widget)
	 */

	/* Important Widgets */
	GtkWidget *top_vbox;
	GtkWidget *whois_FIXME;
	GtkWidget *top_pane;
	GtkWidget *output_hbox;
	GtkWidget *output_box;
	GtkWidget *user_list;
	GtkWidget *user_bbox;
	GtkWidget *nick_label;
	GtkWidget *entry_box;

	/* Callback to change the title of the parent window or notebook etc */
	ChatTitleFn title_cb;
	gpointer    title_cb_data;

	/* Server we are bound to */
	Say2Server *server;

	/* Recipient of any messages typed into the entry box */
	ChatRecipType  recip_type;
	Say2Client    *recip;
};


/* Create a new, empty ChatInterfaces */
ChatInterface *chat_interface_new(void);

/* Set the title callback of a chat interface */
void chat_interface_set_title_cb(ChatInterface *, ChatTitleFn, gpointer);

/* Get an interface, trying to find appropriate one to use or if call else
 * fails, creating a new one somewhere.  Also a function if you already know
 * which interface you want to set.
 */
ChatInterface *chat_interface_get(Say2Server *, ChatRecipType, Say2Client *);
void           chat_interface_set(ChatInterface *, Say2Server *, ChatRecipType,
				  Say2Client *);

/* Disconnect a chat interface making it free for further use */
void chat_interface_disconnect(ChatInterface *);

/* Update relevant chat interfaces information about a particular client */
void chat_interface_update(Say2Server *, ChatUpdateType, Say2Client *);

/* Output formatted information */
void chat_interface_say(Say2Server *, const gchar *, ...);
void chat_interface_sayto(ChatInterface *, const gchar *, ...);

#endif /* !_MEKO_CHAT_INTERFACE_H */
