/* meko
 * Copyright (C) 2001 Scott James Remnant <scott@netsplit.com>.
 * All Rights Reserved.
 *
 * main.c
 *  - Initialisation and main loop
 * -- 
 * $Id: main.c,v 1.13 2001/06/16 20:31:59 scott Exp $
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <meko.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <gnome.h>

#include "meko_app.h"
#include "connect_dialog.h"

/* Forward prototypes */
static void main_die(GnomeClient *, gpointer);
static gint main_save_session(GnomeClient *, gint, GnomeSaveStyle, gint,
			      GnomeInteractStyle, gint, gchar *);

/* Extra options that can be specified on the command line */
static char *geometry = NULL;
static const struct poptOption options[] = {
	{
		"geometry", 0, POPT_ARG_STRING, &geometry, 0,
		N_("Specify the geometry of the main window"),
		N_("GEOMETRY")
	},
	{ NULL, 0, 0, NULL, 0, NULL, NULL }
};


/* main
 * The main function.  Initialises everything that needs to be set up,
 * creates the application window then enters the main loop and stays there
 * until we die.
 */
int
main(int argc, char *argv[])
{
	GnomeClient *client;
	GtkWidget   *app, *dialog;

	/* Initialise Internationalisation */
	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
	textdomain(PACKAGE);
	
	/* Initialise GNOME, GTK+ and all the other gubbins */
	gnome_init_with_popt_table(APPNAME, VERSION, argc, argv,
				   options, 0, NULL);

	/* Session Management is good, m'kay */
	client = gnome_master_client();
	gtk_signal_connect(GTK_OBJECT(client), "save_yourself",
			   GTK_SIGNAL_FUNC(main_save_session), NULL);
	gtk_signal_connect(GTK_OBJECT(client), "die",
			   GTK_SIGNAL_FUNC(main_die), NULL);

	/* Create the main window */
	app = meko_app_new(geometry);
	meko_app_show(app);

	/* Create a connection dialog for our first connection */
	dialog = connect_dialog_new(app);
	gtk_widget_show(dialog);

	/* Main loop, wheee! */
	gtk_main();

	/* That's all folks! */
	return 0;
}

/* main_die
 * Called when the session manager tells us to die.  Leave gtk_main and
 * therefore terminate the program.
 */
static void
main_die(GnomeClient *client, gpointer data)
{
	gtk_main_quit();
}

/* main_save_session
 * Called when the session manager wants us to save our session
 */
static gint
main_save_session(GnomeClient *client, gint phase, GnomeSaveStyle save_style,
		  gint is_shutdown, GnomeInteractStyle interact_style,
		  gint is_fast, gchar *argv0)
{
	gchar **argv;
	gint    argc;

	/* Allocate space for arguments, first is our argv[0] */
	argv = g_new0(gchar *, 2);
	argv[0] = argv0;
	argc = 1;

	/* Set the clone and restart commands */
	gnome_client_set_clone_command(client, argc, argv);
	gnome_client_set_restart_command(client, argc, argv);

	return TRUE;
}
