/* meko
 * Copyright (C) 2001 Scott James Remnant <scott@netsplit.com>.
 * All Rights Reserved.
 *
 * say2_types.h
 * -- 
 * $Id: say2_types.h,v 1.3 2001/06/13 11:53:40 scott Exp $
 * 
 * This file is distributed according to the GNU General Public
 * License.  For full details, read the top of 'main.c' or the
 * file called COPYING that was distributed with this code.
 */

#ifndef _MEKO_SAY2_TYPES_H
#define _MEKO_SAY2_TYPES_H

/* Required includes */
#include <glib.h>
#include <say2/say2.h>

/* Status clients can be on the server */
typedef enum {
	SAY2_STATUS_ACTIVE,
	SAY2_STATUS_BUSY,
	SAY2_STATUS_AWAY,
	SAY2_STATUS_OTHER
} Say2Status;

/* States of the server link */
typedef enum {
	SAY2_SERVER_CONNECTING,
	SAY2_SERVER_CONNECTED,
	SAY2_SERVER_ANNOUNCED,
	SAY2_SERVER_ACTIVE
} Say2ServerState;

/* Structure that holds information about a client on a say2 server. */
typedef struct _say2_client Say2Client;
struct _say2_client {
	/* Number of references */
	gint ref_count;

	/* Basic information */
	gchar *id;
	gchar *nick;
	gchar *email;
	gchar *tagline;

	/* What they are doing */
	Say2Status  status;
	gchar      *status_msg;

	/* Icon */
	char *icon;
};

/* Structure that holds information about a server connection */
typedef struct _say2_server Say2Server;
struct _say2_server {
	/* Number of references */
	gint ref_count;

	/* Socket to talk with it on */
	Socket *sock;

	/* State of the socket */
	Say2ServerState state;

	/* Server name and port and the password we used to connect */
	gchar *server_name;
	gint   server_port;
	gchar *server_pass;

	/* Information about ourselves.  This is the same pointer as our
	 * entry in any given users table.  It's not unsafe to do this because
	 * things aren't freed until their ref_count = 0 :)  [thanks to
	 * whichever GTK+ developer thought of that]
	 */
	Say2Client *me;

	/* Information about people on the channel (maybe including us) */
	GHashTable *users;

	/* Squelch counters.  When these are 0 the responses aren't
	 * taken as anything special and can be sent to the user.  When > 0
	 * it means meko itself has requested something and doesn't want the
	 * user to see the response.
	 *
	 * This still isn't ideal, as it means that if the user does a command
	 * and meko does the same command before the user's reply comes back,
	 * the users will be squelched, not meko's.
	 *
	 * FIXME perhaps a more specific system like a squelch list we can
	 * look up obvious things in:  squelch ( "members" "id:#foo" )
	 * that'd mean that if the user did the same command, it wouldn't
	 * really matter that we squelched it cause there'd be an exact
	 * duplicate following it.
	 */
	gint squelch_prefs;
	gint squelch_members;
	gint squelch_whois;

	/* Should the members command send whois for each entry returned.
	 * Used to reap an initial channel members list and fill the user
	 * list.
	 *
	 * Why not list_clients?  Well Robey won't guarantee that we get
	 * them all returned, and we can't query icons with it either, and
	 * we want icons, we're a gui after all.
	 */
	gint members_do_whois;

	/* Are we in a history (frig with timestamps if they're old) */
	gboolean in_history;

	/* Captured whom information.  Gathered up and output in one go
	 * to make it neater.
	 */
	gboolean in_whom;
	GSList  *whom_list;
	glong    whom_count;
	glong    whom_total;
};

#endif /* !_MEKO_SAY2_TYPES_H */
